@@ -1,4 +1,4 @@
-// Copyright © 2018 Thomas Nagler
+// Copyright © 2021 Thomas Nagler
 //
 // This file is part of the RcppThread and licensed under the terms of
 // the MIT license. For a copy, see the LICENSE.md file in the root directory of
@@ -9,6 +9,6 @@
 #include "RcppThread/RMonitor.hpp"
 #include "RcppThread/Rcout.hpp"
 #include "RcppThread/Thread.hpp"
-#include "RcppThread/Batch.hpp"
 #include "RcppThread/ThreadPool.hpp"
 #include "RcppThread/parallelFor.hpp"
+#include "RcppThread/Progress.hpp"
