@@ -62,7 +62,7 @@ class ThreadPool
     size_t getNumThreads() const;
 
   private:
-    quickpool::ThreadPool pool_;
+    std::unique_ptr<quickpool::ThreadPool> pool_;
     std::thread::id owner_thread_;
 };
 
@@ -75,7 +75,7 @@ inline ThreadPool::ThreadPool()
 //! @param nWorkers number of worker threads to create; if `nWorkers = 0`, all
 //!    work pushed to the pool will be done in the main thread.
 inline ThreadPool::ThreadPool(size_t nWorkers)
-  : pool_{ nWorkers }
+  : pool_{ new quickpool::ThreadPool(nWorkers) }
   , owner_thread_{ std::this_thread::get_id() }
 {}
 
@@ -108,7 +108,7 @@ template<class F, class... Args>
 void
 ThreadPool::push(F&& f, Args&&... args)
 {
-    pool_.push(std::bind(std::forward<F>(f), std::forward<Args>(args)...));
+    pool_->push(std::bind(std::forward<F>(f), std::forward<Args>(args)...));
 }
 
 //! pushes jobs returning a value to the thread pool.
@@ -122,7 +122,7 @@ auto
 ThreadPool::pushReturn(F&& f, Args&&... args)
   -> std::future<decltype(f(args...))>
 {
-    return pool_.async(
+    return pool_->async(
       std::bind(std::forward<F>(f), std::forward<Args>(args)...));
 }
 
@@ -169,7 +169,7 @@ ThreadPool::parallelFor(int begin, int end, F f, size_t nBatches)
     if (nBatches == 0) {
         // each worker has its dedicated range, but can steal part of another
         // worker's ranges when done with own
-        auto thr = std::max(pool_.get_active_threads(), static_cast<size_t>(1));
+        auto thr = std::max(pool_->get_active_threads(), static_cast<size_t>(1));
         auto workers = quickpool::loop::create_workers<F>(f, begin, end, thr);
         for (size_t k = 0; k < thr; k++) {
             this->push([=] { workers->at(k).run(workers); });
@@ -181,17 +181,12 @@ ThreadPool::parallelFor(int begin, int end, F f, size_t nBatches)
             return;
         nBatches = std::min(nBatches, nTasks);
 
-        size_t minSize = nTasks / nBatches;
-        int remSize = nTasks % nBatches;
-
+        size_t sz = nTasks / nBatches;
+        int rem = nTasks % nBatches;
         for (size_t b = 0; b < nBatches; b++) {
-            int bSize = minSize + (remSize-- > 0);
-            this->push([=] {
-                for (int i = begin; i < begin + bSize; ++i) {
-                    f(i);
-                }
-            });
-            begin += bSize;
+            int bs = sz + (rem-- > 0);
+            this->push([=] { for (int i = begin; i < begin + bs; ++i) f(i); });
+            begin += bs;
         }
     }
 }
@@ -235,11 +230,11 @@ ThreadPool::wait()
         return;
     }
     do {
-        pool_.wait(100);
+        pool_->wait(100);
         Rcout << "";
         checkUserInterrupt();
 
-    } while (!pool_.done());
+    } while (!pool_->done());
     Rcout << "";
 }
 
@@ -247,7 +242,7 @@ ThreadPool::wait()
 inline void
 ThreadPool::join()
 {
-    pool_.wait();
+    pool_->wait();
 }
 
 //! sets the number of active threads in the pool.
@@ -255,14 +250,14 @@ ThreadPool::join()
 inline void
 ThreadPool::setNumThreads(size_t threads)
 {
-    pool_.set_active_threads(threads);
+    pool_->set_active_threads(threads);
 }
 
 //! gets the number of active threads in the pool.
 inline size_t
 ThreadPool::getNumThreads() const
 {
-    return pool_.get_active_threads();
+    return pool_->get_active_threads();
 }
 
 } // end namespace RcppThread
