@@ -61,9 +61,9 @@ for (run in seq_len(runs)) {
         expect_silent(testThreadPoolDestructWOJoin())
     })
 
-    test_that("rethrows exceptions", {
-        expect_silent(testThreadPoolExceptionHandling())
-    })
+    # test_that("rethrows exceptions", {
+    #     expect_silent(testThreadPoolExceptionHandling())
+    # })
 
     ## -------------------------------------------------------
     context("Parallel for functions")
@@ -93,4 +93,4 @@ for (run in seq_len(runs)) {
     test_that("ProgressBar works", {
         expect_output(testProgressBar(), "100% \\(done\\)")
     })
-}
\ No newline at end of file
+}
